package com.tocea.easycoverage.framework.junit;

import java.lang.reflect.Method;
import java.util.Set;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;

import junit.framework.TestSuite;

import com.tocea.easycoverage.framework.CheckersRunner;
import com.tocea.easycoverage.framework.ClassChecker;
import com.tocea.easycoverage.framework.MethodChecker;
import com.tocea.easycoverage.framework.api.IInstanceProvider;
import com.tocea.easycoverage.framework.utils.Utils;

public class JUnitTestSuiteProvider {

	private int						timeout	= 0;
	private final CheckersRunner	runner	= new CheckersRunner();

	public TestSuite getTestSuite() {
		return getTestSuite("Easycoverage Test suite");
	}

	public TestSuite getTestSuite(final String name) {
		final TestSuite suite = new TestSuite(name);

		final ExecutorService executorService = Executors.newCachedThreadPool();

		for (final Class<?> clazz : getClasses()) {
			final TestSuite classSuite = new TestSuite(clazz.getName());
			for (final ClassChecker checker : runner.getCheckers(clazz))
				classSuite.addTest(new EasycoverageTestCase(checker,
						executorService, timeout));
			for (final Method method : Utils.getDeclaredPublicMethods(clazz)) {
				final TestSuite methodSuite = new TestSuite(
						Utils.getQualifiedName(method));
				for (final MethodChecker checker : runner.getCheckers(method))
					methodSuite.addTest(new EasycoverageTestCase(checker,
							executorService, timeout));
				if (methodSuite.countTestCases() > 0)
					classSuite.addTest(methodSuite);
			}
			if (classSuite.countTestCases() > 0)
				suite.addTest(classSuite);
		}

		return suite;
	}

	public void addClass(final Class<?> clazz) {
		getClasses().add(clazz);
	}

	public void addClassChecker(final Class<? extends ClassChecker> checker) {
		getClassCheckers().add(checker);
	}

	public void addMethodChecker(final Class<? extends MethodChecker> checker) {
		getMethodCheckers().add(checker);
	}

	public IInstanceProvider getProvider() {
		return runner.getProvider();
	}

	public void setProvider(final IInstanceProvider provider) {
		runner.setProvider(provider);
	}

	public Set<Class<?>> getClasses() {
		return runner.getClasses();
	}

	public void setClasses(final Set<Class<?>> classes) {
		runner.setClasses(classes);
	}

	public int getTimeout() {
		return timeout;
	}

	public void setTimeout(final int timeout) {
		this.timeout = timeout;
	}

	public Set<Class<? extends ClassChecker>> getClassCheckers() {
		return runner.getClassCheckers();
	}

	public void setClassCheckers(
			final Set<Class<? extends ClassChecker>> classCheckers) {
		runner.setClassCheckers(classCheckers);
	}

	public Set<Class<? extends MethodChecker>> getMethodCheckers() {
		return runner.getMethodCheckers();
	}

	public void setMethodCheckers(
			final Set<Class<? extends MethodChecker>> methodCheckers) {
		runner.setMethodCheckers(methodCheckers);
	}

}
